﻿using System;
using System.Collections.Generic;
using System.Linq;
using Data.Interfaces;
using Data.Models;
using Dapper;
using System.Data.SqlClient;
using Data;
using System.Threading.Tasks;
using Data.Attributes;
using System.Dynamic;

namespace Data.Repositories
{
    public class BaseRepository<T> : IRepository<T> where T : BaseModel
    {

        public async virtual Task<IEnumerable<T>> GetAll(long CompanyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();

                //if(typeof(T) is typeof(CompanyRelatedModel)){};

                //if (typeof(T).Name==0){}
                var properties = typeof(T).GetProperties();
                string command = "";

                if (Array.Exists(properties, col => col.Name == "CompanyId"))
                {
                    command = String.Format("SELECT * FROM [{0}] WHERE CompanyId = @CompanyId", typeof(T).Name);
                    return await Task.FromResult<IEnumerable<T>>(connection.Query<T>(@command, new { CompanyId = CompanyId }));
                }
                else 
               {
                     command = String.Format("SELECT * FROM [{0}]", typeof(T).Name);
                     return await Task.FromResult<IEnumerable<T>>(connection.Query<T>(@command));
                }
                
            }

        }

        public async virtual Task<T> GetById(long id)
        {

            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string tableName = typeof(T).Name;
                string command = string.Format("SELECT * FROM [{0}] WHERE Id = @Id", tableName);
                var item = connection.Query<T>(@command, new { Id = id }).FirstOrDefault();
                var properties = item.GetType().GetProperties().Where(p => p.GetCustomAttributes(typeof(ForeignKeys), false).Any());

                foreach (var prop in properties)
                {
                    if (prop.PropertyType != typeof(IEnumerable<long>))
                    {
                        continue;
                    }

                    string sndTableName = string.Format("{0}To{1}", tableName, GetTableByAttribute());
                    string idName = string.Format("{0}Id", tableName);

                    command = string.Format("SELECT * FROM [{0}] WHERE {1} = @Id", sndTableName, idName);
                    prop.SetValue(prop, connection.Query(command, new { Id = id }));
                }

                return await Task.FromResult<T>(item);
            }

        }

        public async virtual Task<long> Create(T model)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                var properties = typeof(T)
                    .GetProperties();

                string tableName = model.GetType().Name;

                List<string> columns = 
                    properties
                    .Where(p => p.Name != "Id" && 
                           !p.GetCustomAttributes(false).Any(a => 
                               a.GetType() == typeof(IgnoreDB) || 
                               a.GetType() == typeof(ForeignKeys)))
                    .Select(p => p.Name)
                    .ToList();

                model.CreatedAt = DateTimeOffset.Now;
                string command = string.Format("INSERT INTO [{0}] ({1}) VALUES ({2}); SELECT CAST(SCOPE_IDENTITY() AS int)",
                    tableName,
                    string.Join(", ", columns),
                    string.Join(", ", columns.Select(p => string.Format("@{0}", p))));

                var id = connection.Query<long>(@command, model).Single();

                // update help table 1:n, m:n
                var foreignColumns = properties.Where(p => p.GetCustomAttributes(typeof(ForeignKeys), false).Any());
                foreach (var column in foreignColumns)
                {
                    if (column.PropertyType == typeof(IEnumerable<long>))
                    {
                        foreach (var sndId in column as IEnumerable<long>)
                        {
                            string sndTable = GetTableByAttribute();
                            var columnNames = new List<string> 
                            { 
                                string.Format("{0}Id", tableName), 
                                string.Format("{0}Id", sndTable) 
                            };

                            var data = new ExpandoObject() as IDictionary<string, object>;
                            data.Add(columnNames[0], id);
                            data.Add(columnNames[1], sndId);

                            command = string.Format("INSERT INTO [{0}To{1}] ({2}) VALUES ({3})", 
                                tableName, 
                                sndTable, 
                                columnNames, 
                                columnNames.Select(c => string.Format("@{0}", c)));

                            connection.Query(command, data);
                        }
                    }
                }

                return await Task.FromResult<long>(id);
            }
        }

        public async virtual Task<long> Update(T model)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();

                if (model.Id > 0)
                {
                    List<string> columns = typeof(T)
                        .GetProperties()
                        .Where(p => p.Name != "Id" && !p.GetCustomAttributes(typeof(IgnoreDB), false).Any())
                        .Select(p => string.Format("{0} = @{0}", p.Name))
                        .ToList();

                    string command = string.Format("UPDATE [{0}] SET {1} WHERE Id = @Id", typeof(T).Name, string.Join(", ", columns));
                    connection.Query<long>(@command, model);
                }
                else
                {
                    return await Create(model);
                }

                return await Task.FromResult<long>(model.Id);
            }
        }

        public async virtual Task Delete(long id)
        {

            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = String.Format("DELETE FROM [{0}] WHERE Id = @Id", typeof(T).Name);
                connection.Execute(@command, new { Id = id });
            }

            await Task.FromResult<object>(null);
        }

        #region PRIVATE

        private string GetTableByAttribute()
        {
            return (Attribute.GetCustomAttribute(typeof(T), typeof(ForeignKeys)) as ForeignKeys).Name;
        }

        #endregion
    }
}
