﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Data.Models;
using Data.Interfaces;
using System.Data.SqlClient;
using Dapper;
using Data.Attributes;

namespace Data.Repositories
{
    public static class CostPlanRepository
    {
        public static async Task<IEnumerable<CostPlan>> GetAllCrp(this IRepository<CostPlan> baseRepo, long CrpId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT * FROM [CostPlan] WHERE CrpId = @id";
                return await Task.FromResult<IEnumerable<CostPlan>>(connection.Query<CostPlan>(@command, new { id = CrpId }));
            }
        }

        public static async Task<IEnumerable<WorkPlan>> LoadWorkPlan(this IRepository<CostPlan> baseRepo, long companyId, DateTimeOffset from, DateTimeOffset to, bool type)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT costPlan.DateCreate as PlanDate, crp.[Description] as CrpName, soilBlock.IdentificationNumber as SoilBlockIdNumber,";
                       command += "soilBlock.Name as SoilBlockName, partSb.[Description] as PartSoilBlockName, partSb.Area as PartSoilBlockArea, machine.Name as MachineryName,";
                       command += "work.Name as WorkName, costPlan.[Status] as CostPlanState, soilBlock.CompanyId";
                       command += " FROM [CropRotationPlan] as crp ";
                       command += "INNER JOIN [CostPlan] as costPlan ON (crp.Id = costPlan.CrpId)";
                       command += "INNER JOIN [WorkCategory] as work ON (costPlan.WorkId = work.Id)";
                       command += "INNER JOIN [Machinery] as machine ON (costPlan.MachineryId = machine.Id)";
                       command += "INNER JOIN [PartSoilBlock] as partSb ON (crp.PartSoilBlockId = partSb.Id)";
                       command += "INNER JOIN [SoilBlock] as soilBlock ON (partSb.SoilBlockId = soilBlock.Id)";
                       command += "WHERE soilBlock.CompanyId=@CompanyId AND costPlan.[Status]=@Type AND (costPlan.DateCreate >= @From AND costPlan.DateCreate <= @To)";
                       return await Task.FromResult<IEnumerable<WorkPlan>>(connection.Query<WorkPlan>(@command, new { CompanyId = companyId, Type = type, From = from, To = to }));
            }
        }
    }
}
