﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Data.Models;
using System.Data.SqlClient;
using Data.Interfaces;
using Dapper;
using Data.Attributes;
using System.Threading;

namespace Data.Repositories
{
    public static class CropRotationPlanRepository
    {
        public static async Task<CropRotationPlan> GetCrpByPartId(this IRepository<CropRotationPlan> baseRepo, long partSoilBlockId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT * FROM [CropRotationPlan] WHERE PartSoilBlockId = @id";
                return await Task.FromResult<CropRotationPlan>(connection.Query<CropRotationPlan>(@command, new { id = partSoilBlockId }).FirstOrDefault());
            }
        }

        public static async Task<IEnumerable<CropRotationPlan>> GetPlansByYear(this IRepository<CropRotationPlan> baseRepo, long year, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT CropRotationPlan.* FROM [CropRotationPlan] JOIN [PartSoilBlock] ON (CropRotationPlan.PartSoilBlockId = PartSoilBlock.Id) WHERE (PartSoilBlock.MarketingYear = @year) AND (CropRotationPlan.CompanyId = @companyId)";
                return await Task.FromResult<IEnumerable<CropRotationPlan>>(connection.Query<CropRotationPlan>(@command, new { year = year, companyId = companyId }));
            }
        }

        public static async Task<IEnumerable<CropRotationPlan>> GetPlansByYearPartSoilBlock(this IRepository<CropRotationPlan> baseRepo, long year, long partSoilBlockId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT CropRotationPlan.* FROM [CropRotationPlan] JOIN [PartSoilBlock] ON (CropRotationPlan.PartSoilBlockId = PartSoilBlock.Id) WHERE (PartSoilBlock.MarketingYear = @year) AND (PartSoilBlock.Id = @partSoilBlockId)";
                return await Task.FromResult<IEnumerable<CropRotationPlan>>(connection.Query<CropRotationPlan>(@command, new { year = year, partSoilBlockId = partSoilBlockId }));
            }
        }

        public static async Task<IEnumerable<ActualCropStatus>> ActualCropStatus(this IRepository<CropRotationPlan> baseRepo, long year, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "select crop.Name, SUM(psb.Area) as TotalArea FROM CropRotationPlan as crp INNER JOIN PartSoilBlock as psb ON crp.PartSoilBlockId = psb.Id INNER JOIN Crop as crop ON crp.CropId = crop.Id WHERE (crp.CompanyId = @companyId) And (psb.MarketingYear = @marketinYear) GROUP BY crop.Name";
                return await Task.FromResult<IEnumerable<ActualCropStatus>>(connection.Query<ActualCropStatus>(@command, new { marketinYear = year, companyId = companyId }));
            }
        }

        public static async Task<IEnumerable<StockAlarmItem>> UseQuantity(this IRepository<CropRotationPlan> baseRepo, long year, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT stock.Id as StockId, stock.Name, SUM(cp.Quantiy) as NeedQuantity, stock.MeasureId FROM CropRotationPlan as crp INNER JOIN CostPlan as cp ON crp.Id = cp.CrpId INNER JOIN PartSoilBlock as pb ON crp.PartSoilBlockId = pb.Id INNER JOIN StockItem as stock ON cp.StockItemId = stock.Id WHERE ((crp.CompanyId = @companyId) And (pb.MarketingYear = @marketinYear) AND (cp.StockItemId <> 0) AND (cp.[Status] = 0)) GROUP BY stock.Id, stock.Name, stock.MeasureId";
                return await Task.FromResult<IEnumerable<StockAlarmItem>>(connection.Query<StockAlarmItem>(@command, new { marketinYear = year, companyId = companyId }));
            }
        }

        public static async Task<IEnumerable<StockAlarmFuel>> UseFuel(this IRepository<CropRotationPlan> baseRepo, long year, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT stock.id as FuelId, stock.Name,SUM((pb.Area * cons.AverageConsumption)) as Consuption, stock.MeasureId FROM CropRotationPlan as crp INNER JOIN CostPlan as cp ON crp.Id = cp.CrpId INNER JOIN PartSoilBlock as pb ON crp.PartSoilBlockId = pb.Id INNER JOIN StockItem as stock ON cp.FuelId = stock.Id INNER JOIN WcatToMachine as cons ON ((cons.MachineId = cp.MachineryId) AND (cons.WorkId = cp.WorkId)) WHERE ((crp.CompanyId = @companyId) And (pb.MarketingYear = @marketinYear) AND (cp.[Status] = 0)) GROUP BY stock.id, stock.Name, stock.MeasureId";
                return await Task.FromResult<IEnumerable<StockAlarmFuel>>(connection.Query<StockAlarmFuel>(@command, new { marketinYear = year, companyId = companyId }));
            }
        }

    }
}
