﻿using Data.Interfaces;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Dapper;

namespace Data.Repositories
{
    public static class InvoiceRepository
    {
        public static async Task<long> GetLastInvoiceNumber(this IRepository<Invoice> baseRepo, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT MAX(InvoiceNumber) as number FROM [Invoice] WHERE CompanyId = @CompanyId";
                var maxNumber = connection.Query(@command, new { CompanyId = companyId }).FirstOrDefault();
                var result = 1;
                if (maxNumber.number != null)
                {
                    result = maxNumber.number + 1;
                }
                return await Task.FromResult(result);
            }
        }

        public static async Task<bool> CheckInvoiceNumber(this IRepository<Invoice> baseRepo, long invoiceNumber, long companyId)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT * FROM [Invoice] WHERE CompanyId = @CompanyId AND InvoiceNumber = @InvoiceNumber";
                var maxNumber = connection.Query<Invoice>(@command, new { CompanyId = companyId, InvoiceNumber = invoiceNumber }).FirstOrDefault();
                if (maxNumber == null)
                {
                    return await Task.FromResult(true);
                }
                else {
                    return await Task.FromResult(false);
                }
            }
        }

        public static async Task<bool> ChangePaySate(this IRepository<Invoice> baseRepo, long invoiceId, bool type)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "UPDATE [Invoice] SET [PayState] = @Type WHERE [ID] = @InvoiceId";
                connection.Query(@command, new { Type = type, InvoiceId = invoiceId });
                return await Task.FromResult(true);
            }
        }

        public static async Task<IEnumerable<Invoice>> GetFalseInvoices(this IRepository<Invoice> baseRepo, long companyId, DateTime from, DateTime to)
        {
            using (SqlConnection connection = ConnectionManager.GetConnection())
            {
                connection.Open();
                string command = "SELECT * FROM [Invoice] WHERE (([CompanyId] = @CompanyId) AND (([PayState]) = 0 OR ([PayDate]>=@From AND [PayDate]<=@To)))";
                return await Task.FromResult<IEnumerable<Invoice>>(connection.Query<Invoice>(@command, new { CompanyId = companyId, From = from, To = to }));
            }
        }
    }
}
