﻿using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using WebAppDp.ViewModels;
using bussines.Services;
using WebAppDp.Helpers;
using System.Web.Http;
using System.Threading.Tasks;
using Data.Enums;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("businessRecord")]
    public class BusinessRecordController : BaseController<BusinessRecord,BusinessRecordViewModel>
    {
        public BusinessRecordService Service { get; set; }

        public BusinessRecordController(BusinessRecordService service) : base(service)
        {
            Service = service;
        }

        [Route("record"), HttpPost]
        public async Task<IEnumerable<BusinessRecordViewModel>> GetAllByType(FromToViewModel model)
        {
            var models = DataToModels(Service.GetAllByType(model.Type, IdentityHelper.CurrentUser.CompanyId, model.From.Split('T')[0], model.To.Split('T')[0]).Result);
            return await Task.FromResult(models);
        }

        [Route("getAllFromTo"), HttpPost]
        public async Task<IEnumerable<BusinessRecordViewModel>> GetAllFromTo(FromToViewModel model)
        {
            var models = DataToModels(Service.GetAllFromTo(IdentityHelper.CurrentUser.CompanyId, model.From.Split('T')[0], model.To.Split('T')[0]).Result);
            return await Task.FromResult(models);
        }


        protected override BusinessRecord ModelToData(BusinessRecordViewModel model)
        {
            return new BusinessRecord 
            { 
                CategoryId = model.CategoryId,
                CompanyId = IdentityHelper.CurrentUser.CompanyId,
                CreatedAt = model.CreatedAt,
                Description = model.Description,
                Id = model.Id,
                MeasureId = model.MeasureId,
                Name = model.Name,
                Price = model.Price,
                Quantity = model.Quantity,
                StockItemId = model.StockItemId,
                VendorId = model.VendorId,
                Income = model.Income,
                HarvestedItemId = model.HarvestedItemId
            };

        }

        protected override BusinessRecordViewModel DataToModel(BusinessRecord data)
        {
            var vendor = new VendorService();

            return new BusinessRecordViewModel
            {
                CategoryId = data.CategoryId,
                CompanyId = data.CompanyId,
                CreatedAt = data.CreatedAt,
                Description = data.Description,
                Id = data.Id,
                MeasureId = data.MeasureId,
                Name = data.Name,
                Price = data.Price,
                Quantity = data.Quantity,
                StockItemId = data.StockItemId,
                VendorId = data.VendorId,
                Income = data.Income,
                HarvestedItemId = data.HarvestedItemId,
                dodavatel = vendor.GetById(data.VendorId).Result,
                categoryName = Enum.GetName(typeof(CategoryType), data.CategoryId),
                measureName = Enum.GetName(typeof(Measure), data.MeasureId)
            };
        }
    }
}