﻿using bussines.Services;
using Data.Attributes;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("costPlan")]
    public class CostPlanController : BaseController<CostPlan,CostPlanViewModel>
    {
        public CostPlanService Service { get; set; }

        public CostPlanController(CostPlanService service)
            :base(service)
        {
        }

        [Route("crp/{crpId}"), HttpGet]
        public async Task<IEnumerable<CostPlanViewModel>> GetAllCategory(long crpId)
        {
            return await Task.Run<IEnumerable<CostPlanViewModel>>(() =>
            {
                var data = Service.GetAllCrp(crpId).Result;
                return this.DataToModels(data);
            });
        }

        [Route("loadWorkPlan"), HttpPost]
        public async Task<IEnumerable<WorkPlan>> LoadWorkPlan(FromToViewModel model) 
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.GetWorkPlan(companyId, model.From, model.To, model.Type).Result);
        }

        protected override CostPlan ModelToData(CostPlanViewModel model)
        {
            DateTimeOffset modelAppDate = model.DateApplication;
            DateTimeOffset modelCreateDate = model.DateCreate;
            if (model.DateApplicationString != null) 
            {
                modelAppDate = DateTimeOffset.Parse(model.DateApplicationString);
            }
            if (model.DateCreateString != null)
            {
                modelCreateDate = DateTimeOffset.Parse(model.DateCreateString);
            }
            return new CostPlan 
            {
                ActualCosts = model.ActualCosts,
                AnticipatedCosts = model.AnticipatedCosts,
                CreatedAt = model.CreatedAt,
                CrpId = model.CrpId,
                Dosage = model.Dosage,
                Id = model.Id,
                Status = model.Status,
                StockItemId = model.StockItemId,
                DateCreate = modelCreateDate,
                MachineryId = model.MachineryId,
                WorkId = model.WorkId,
                Quantiy = model.Quantiy,
                N = model.N,
                S = model.S,
                P = model.P,
                K = model.K,
                FuelId = model.FuelId,
                DateApplication = modelAppDate
            };
        }

        protected override CostPlanViewModel DataToModel(CostPlan data)
        {
            MachineryViewModel machinery = new MachineryViewModel();
            WorkCategoryViewModel workCat = new WorkCategoryViewModel();
            StockItemViewModel stock = new StockItemViewModel();
            StockItemViewModel fuel = new StockItemViewModel();

            StockItemService stockItemService = new StockItemService();

            if (data.MachineryId > 0)
            {
                machinery = new MachineryController().Get(data.MachineryId).Result;
            }
            if (data.WorkId > 0)
            {
                workCat = new WorkCategoryController().Get(data.WorkId).Result;
            }
            if (data.StockItemId > 0)
            {
                stock = new StockItemController(stockItemService).Get(data.StockItemId).Result;
            }
            if (data.FuelId > 0)
            {
                fuel = new StockItemController(stockItemService).Get(data.FuelId).Result;
            }

            return new CostPlanViewModel
            {
                ActualCosts = data.ActualCosts,
                AnticipatedCosts = data.AnticipatedCosts,
                CreatedAt = data.CreatedAt,
                CrpId = data.CrpId,
                Dosage = data.Dosage,
                Id = data.Id,
                Status = data.Status,
                StockItemId = data.StockItemId,
                MachineryId = data.MachineryId,
                WorkId = data.WorkId,
                Quantiy = data.Quantiy,
                N = data.N,
                S = data.S,
                P = data.P,
                K = data.K,
                machinery = machinery,
                stockItem = stock,
                workCategory = workCat,
                openStatus = false,
                FuelId = data.FuelId,
                fuelItem = fuel,
                DateCreate = data.DateCreate,
                DateApplication = data.DateApplication
            };
        }
    }
}