﻿using bussines.Services;
using Data.Attributes;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("crp")]
    public class CrpController : BaseController<CropRotationPlan, CrpViewModel>
    {
        public CropRotationPlanService Service { get; set; }

        public CrpController(CropRotationPlanService service)
            : base(service)
        {
            Service = service;
        }

        [Route("crp/{partSoilBlockId}"), HttpGet]
        public async Task<CrpViewModel> GetCrpByPartId(long partSoilBlockId)
        {
            var model = DataToModel(Service.GetCrpByPartId(partSoilBlockId).Result);
            return await Task.FromResult<CrpViewModel>(model);
        }

        [Route("all/{year}"), HttpGet]
        public async Task<IEnumerable<CrpViewModel>> GetAll(long year) 
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(this.DataToModels(Service.GetPlansByYear(year, companyId).Result));
        }

        [Route("getBy/{year}/part/{partSoilBlockId}"), HttpGet]
        public async Task<IEnumerable<CrpViewModel>> GetBy(long year, long partSoilBlockId) 
        {
            var models = Service.GetPlansByYearPartSoilBlockId(year, partSoilBlockId).Result;
            return await Task.FromResult(DataToModels(models));
        }

        [Route("actualCropStatus/{year}"), HttpGet]
        public async Task<IEnumerable<ActualCropStatus>> ActualCropStatus(long year)
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.ActualCropStatus(year, companyId).Result);
        }

        [Route("stockMaterialAlarm/{year}"), HttpGet]
        public async Task<IEnumerable<StockAlarmItem>> StockMaterialAlarm(long year) 
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.StockMaterialAlarm(year, companyId).Result);
        }

        [Route("stockFuelAlarm/{year}"), HttpGet]
        public async Task<IEnumerable<StockAlarmFuel>> StockFuelAlarm(long year)
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.StockFuelAlarm(year, companyId).Result);
        }

        public IEnumerable<CrpViewModel> DataToModels(IEnumerable<CropRotationPlan> data)
        {
            return data.Select(p => DataToModel(p));
        }

        protected override CropRotationPlan ModelToData(CrpViewModel model)
        {
            return new CropRotationPlan
            {
                AverageYield = model.AverageYield,
                Costs = model.Costs,
                CreatedAt = model.CreatedAt,
                CropId = model.CropId,
                DateChange = model.DateChange,
                Description = model.Description,
                Id = model.Id,
                PartSoilBlockId = model.PartSoilBlockId,
                CompanyId = IdentityHelper.CurrentUser.CompanyId,
                RealCost = model.RealCost,
                HarvestedStatus = model.HarvestedStatus,
                HarvestedQuantity = model.HarvestedQuantity
            };
        }

        protected override CrpViewModel DataToModel(CropRotationPlan data)
        {
            var partSBCtrol = new PartSoilBlockController(new PartSoilBlockService());
            var cropCtrol = new CropController();
            if (data == null) {
                return null;
            } else {
                return new CrpViewModel
                {
                    AverageYield = data.AverageYield,
                    Costs = data.Costs,
                    CreatedAt = data.CreatedAt,
                    CropId = data.CropId,
                    DateChange = data.DateChange,
                    Description = data.Description,
                    Id = data.Id,
                    PartSoilBlockId = data.PartSoilBlockId,
                    CompanyId = data.CompanyId,
                    partSoilBlock = partSBCtrol.Get(data.PartSoilBlockId).Result, //partSoilBlockService.GetById(data.PartSoilBlockId).Result
                    crop = cropCtrol.Get(data.CropId).Result,
                    RealCost = data.RealCost,
                    HarvestedStatus = data.HarvestedStatus,
                    HarvestedQuantity = data.HarvestedQuantity
                };
            };
        }
    }
}