﻿using bussines.Services;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("invoice")]
    public class InvoiceController : BaseController<Invoice, InvoiceViewModel>
    {
        InvoiceService Service { get; set; }
        public InvoiceController(InvoiceService service) 
            : base(new InvoiceService())
        {
            Service = service;
        }

        [Route("getLastInvoiceNumber"), HttpGet]
        public async Task<long> GetLastInvoiceNumber() 
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.GetLastInvoiceNumber(companyId).Result);
        }

        [Route("checkInvoiceNumber/{invoiceNumber:long}"), HttpGet]
        public async Task<bool> CheckInvoiceNumber(long invoiceNumber)
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            return await Task.FromResult(Service.CheckInvoiceNumber(invoiceNumber, companyId).Result);
        }

        [Route("saveNewInvoice"), HttpPost]
        public async Task<long> SaveNewInvoice(InvoiceViewModel model)
        {
            var data = this.ModelToData(model);
            return await Task.FromResult(Service.CreateNewInvoice(data, model.BankAccount,model.BankAccount.ConstSym, model.BankAccount.VariableSym, model.BankAccount.SpecSym).Result);
        }

        [Route("setStateTrue/{invoiceId:long}"), HttpGet]
        public async Task<bool> SetStateTrue(long invoiceId)
        {
            return await Task.FromResult(Service.ChangePayState(invoiceId, true).Result);
        }

        [Route("setStateFalse/{invoiceId:long}"), HttpGet]
        public async Task<bool> SetStateFalse(long invoiceId)
        {
            return await Task.FromResult(Service.ChangePayState(invoiceId, false).Result);
        }

        [Route("getAllFalseInvoice"), HttpPost]
        public async Task<IEnumerable<InvoiceViewModel>> GetAllFalseInvoice(FromToViewModel data)
        {
            return await Task.FromResult(this.DataToModels(Service.GetAllFalseInvoices(IdentityHelper.CurrentUser.CompanyId, data.From, data.To).Result));
        }

        protected override Invoice ModelToData(InvoiceViewModel model)
        {
            return new Invoice {
                Id = model.Id,
                CreatedAt = model.CreatedAt,
                BankTransactionId = model.BankTransactionId,
                CompanyId = IdentityHelper.CurrentUser.CompanyId,
                InvoiceItemsCSV = model.InvoiceItemsCSV,
                InvoiceNumber = model.InvoiceNumber,
                InvoiceType = model.InvoiceType,
                PayDate= model.PayDate,
                PayType= model.PayType,
                VendorId = model.VendorId,
                CustomerId = model.CustomerId,
                TotalCost = model.TotalCost,
                PayState = model.PayState
            };
        }

        protected override InvoiceViewModel DataToModel(Invoice data)
        {
            VendorService vendorService = new VendorService();
            BankTransactionService transactionService = new BankTransactionService();
            VendorViewModel vendorName = null;
            VendorViewModel customerName = null;
            BankTransaction bankTransaction = null;
            if (data.VendorId > 0) {
                vendorName = VendorHelper.DataToModel(vendorService.GetById(data.VendorId).Result);
            };
            if (data.CustomerId > 0)
            {
                customerName = VendorHelper.DataToModel(vendorService.GetById(data.CustomerId).Result);
            };
            if (data.BankTransactionId > 0) {
                bankTransaction = transactionService.GetById(data.BankTransactionId).Result;
            }

            string _payState = "Nedefinováno";
            if (data.PayState) 
            {
                _payState = "Uhrazeno";
            } else {
                _payState = "Neuhrazeno";
            };


            return new InvoiceViewModel
            {
                Id = data.Id,
                CreatedAt = data.CreatedAt,
                BankTransactionId = data.BankTransactionId,
                CompanyId = data.CompanyId,
                InvoiceItemsCSV = data.InvoiceItemsCSV,
                InvoiceNumber = data.InvoiceNumber,
                InvoiceType = data.InvoiceType,
                PayDate = data.PayDate,
                PayType = data.PayType,
                VendorId = data.VendorId,
                CustomerId = data.CustomerId,
                TotalCost = data.TotalCost,
                Vendor = vendorName,
                Customer = customerName,
                BankTransaction = bankTransaction,
                PayState = data.PayState,
                PayStateName = _payState
            };
        }
    }
}
