﻿using business;
using bussines.Services;
using Data.Enums;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("stock")]
    public class StockItemController : BaseController<StockItem, StockItemViewModel>
    {
        public StockItemService Service { get; set; }

        public StockItemController(StockItemService service)
            : base(service)
        {
            Service = service;
        }
        
        [Route("newStockItem"), HttpPost]
        public async Task<long> PostNewStockItem(StockItemViewModel model)
        {
            return await Service.PutStockItemWithRelation(this.ModelToData(model), model.cropRelation);
        }

        [Route("category/{CategoryId}"), HttpGet]
        public async Task<IEnumerable<StockItem>> GetAllCategory(long CategoryId)
        {
            var data = Service.GetAllCategory(CategoryId,IdentityHelper.CurrentUser.CompanyId).Result;
            return await Task.Run(() => this.DataToModels(data));
        }

        [Route("status"), HttpGet]
        public async Task<IEnumerable<StockStatus>> GetStockStatus()
        {
            return await Service.GetStockStatus(IdentityHelper.CurrentUser.CompanyId);
        }

        [Route("category/{categoryId}/relation/{RelationId}"), HttpGet]
        public async Task<IEnumerable<StockItem>> GetCategoryByRelation(long categoryId, long RelationId)
        {
            var data = Service.GetCategoryByRelation(categoryId, RelationId).Result;
            return await Task.Run(() => this.DataToModels(data));
        }

        protected override StockItem ModelToData(StockItemViewModel model)
        {
            return new StockItem
            {
                CategoryId = model.CategoryId,
                CompanyId = IdentityHelper.CurrentUser.CompanyId,
                //CreatedAt = model.CreatedAt,
                Description = model.Description,
                Id = model.Id,
                Name = model.Name,
                Price = model.Price,
                Quantity = model.Quantity,
                DateChange = DateTime.UtcNow,
                Dosage = model.Dosage,
                MeasureId = model.MeasureId,
                N = model.N,
                S = model.S,
                P = model.P,
                K = model.K
            };
        }

        protected override StockItemViewModel DataToModel(StockItem data)
        {
            float unitPrice = 0;
            if ((data.Price == 0) || (data.Quantity == 0))
            {
                unitPrice = 0;
            }
            else 
            {
                unitPrice = data.Price / data.Quantity;
            }
            return new StockItemViewModel
            {
                CategoryId = data.CategoryId,
                CompanyId = data.CompanyId,
                CreatedAt = data.CreatedAt,
                DateChange = data.DateChange,
                Description = data.Description,
                Id = data.Id,
                Name = data.Name,
                Price = data.Price,
                Quantity = data.Quantity,
                Dosage = data.Dosage,
                MeasureId = data.MeasureId,
                N = data.N,
                S = data.S,
                P = data.P,
                K = data.K,
                categoryName = Enum.GetName(typeof(CategoryType), data.CategoryId),
                measureName = Enum.GetName(typeof(Measure), data.MeasureId),
                priceUnit = unitPrice

            };
        }
    }
}
