﻿using System;
using System.Threading.Tasks;
using Microsoft.Owin.Security.OAuth;
using System.Security.Claims;
using Data.Interfaces;
using Data.Repositories;
using Data.Models;
using Data;
using System.Threading;



namespace WebAppDp.Providers
{
    public class ApplicationOAuthProvider : OAuthAuthorizationServerProvider
    {
        public override async Task ValidateClientAuthentication(OAuthValidateClientAuthenticationContext context)
        {
            await Task.Run(() => context.Validated());
        }

        public override async Task ValidateAuthorizeRequest(OAuthValidateAuthorizeRequestContext context)
        {
            await Task.Run(() => context.Validated());
        }

        public override async Task GrantResourceOwnerCredentials(OAuthGrantResourceOwnerCredentialsContext context)
        {
            await Task.Run(() =>
            {
                context.OwinContext.Response.Headers.Add("Access-Control-Allow-Origin", new[] { "*" });

                IRepository<User> repository = new BaseRepository<User>();

                User user = repository.CheckCreditials(context.UserName, Utils.GetHash(context.Password)).Result;


                if (user == null)
                {
                    context.SetError("invalid_grant", "The user name or password is incorrect.");
                    //return;
                }
                else
                {

                    var identity = new ClaimsIdentity(context.Options.AuthenticationType);

                    identity.AddClaim(new Claim("sub", context.UserName));
                    identity.AddClaim(new Claim("role", ((int)user.Role).ToString()));
                    identity.AddClaim(new Claim("companyId", user.CompanyId.ToString()));
                    identity.AddClaim(new Claim("userId", user.Id.ToString()));

                    //var id = Thread.CurrentPrincipal(user.Id);
                    //context.Response.Set<string>("user_id", user.Id.ToString());
                    context.Response.Headers.Add("user_id", new string[] { user.Id.ToString() });
                    context.Validated(identity);
                }
            });

        }
    }
}