﻿'use strict'

dpApp.controller('financeController',
    function ($scope, $location, cropService, $modal, vendorService, settingsService, businessRecordService, invoiceService, confirmModalFactory, addressService, contactService)
    {
        $scope.vendorDetail = false;
        $scope.invoiceDetail = false;
        $scope.businessRecordFrom = new Date(new Date().setMonth(new Date().getMonth() - 1));
        $scope.businessRecordTo = new Date();
        //modal action
        $scope.showNewVendorModal = function () {
            var modalInstance = $modal.open({
                templateUrl: '/Scripts/_app/views/modals/newVendorModal.html',
                controller: 'newVendorModalController',
                size: 1,
                resolve: {
                    data: function () {
                        return [];
                    }
                }
            });

            modalInstance.result.then(function () {
                $scope.loadVendorList();
            });
        };


        $scope.hideInvoiceDetail = function () {
            $scope.invoiceDetail = false;
        };

        $scope.loadVendorList = function ()
        {
            $scope.vendorLoader = true;
            $scope.editAddress = false;
            $scope.editContact = false;
            $scope.editBankAccount = false;
            $scope.hideInvoiceDetail();
            vendorService.getAll(
                function (data)
                {
                    console.log("get all vendor");
                    console.log(data.plain());
                    $scope.vendorTable = data.plain();
                    $scope.vendorLoader = false;
                    $scope.vendorDetail = false;
                },
                function (err)
                {
                    console.log("Error load vendor list.");
                    console.log(err);
                });
        }
        $scope.loadVendorList();

        $scope.loadBusinessRecordList = function () {
            $scope.hideInvoiceDetail();
            $scope.businessRecordLoader = true;
            $scope.vendorDetail = false;
            var obj = {
                From: document.getElementById("businessRecordFrom").value,
                To: document.getElementById("businessRecordTo").value,
                Type: false
            };
            businessRecordService.getAllFromTo(obj,
                function (data)
                {
                    console.log("all business records");
                    console.log(data.plain());
                    $scope.businessRecordTable = data.plain();
                    $scope.businessRecordLoader = false;
                },
                function (err)
                {
                    console.log(err);
                });
        }
        
        $scope.loadBusinessRecordType = function (type) {
            $scope.hideInvoiceDetail()
            $scope.businessRecordLoader = true;
            $scope.vendorDetail = false;
            var obj = {
                From: document.getElementById("businessRecordFrom").value,
                To: document.getElementById("businessRecordTo").value,
                Type: type
            };
            businessRecordService.getAllByType(obj,
                function (data) {
                    console.log("all business records");
                    console.log(data.plain());
                    $scope.businessRecordTable = data.plain();
                    $scope.businessRecordLoader = false;
                },
                function (err) {
                    console.log(err);
                });
        }

        $scope.loadInvoices = function () {
            $scope.invoicesLoader = true;
            $scope.vendorDetail = false;
            $scope.hideInvoiceDetail();
            invoiceService.getInvoices(
                function (data) {
                    console.log("all invoices records");
                    console.log(data.plain());
                    $scope.invoicesTable = data.plain();
                    $scope.invoicesLoader = false;
                },
                function () { });
        }

        $scope.createInvoice = function () {
            $location.path('/app/invoice');
        }

        $scope.getVendorDetail = function (vendorItem) {
            $scope.vendorDetailItem = vendorItem;
            $scope.vendorDetail = true;
        };

        $scope.hideVendorDetail = function () {
            $scope.vendorDetail = false;
        }

        $scope.showAddBankAccountModal = function () {
            var modalInstance = $modal.open({
                templateUrl: '/Scripts/_app/views/modals/addBankAccountModal.html',
                controller: 'addBankAccountModalController',
                size: 'md',
                resolve: {
                    vendorData: function () {
                        return $scope.vendorDetailItem;
                    }
                }
            });
            modalInstance.result.then(function () {
                $scope.loadVendorList();
            });
        }

        $scope.showEditAddress = function () {
            $scope.editAddress = true;
        }

        $scope.hideEditAddress = function () {
            $scope.editAddress = false;
            $scope.loadVendorList();
        }

        $scope.saveEditAddress = function (model) {
            confirmModalFactory('md', function () {
                addressService.put(model,
                    function (data) {
                        $scope.loadVendorList();
                    },
                    function (err) {
                        $scope.loadVendorList();
                    });
            });
        };

        $scope.showEditContact = function () {
            $scope.editContact = true;
        }

        $scope.hideEditContact = function () {
            $scope.editContact = false;
            $scope.loadVendorList();
        }

        $scope.saveEditContact = function (model) {
            confirmModalFactory('md', function () {
                contactService.put(model,
                    function (data) {
                        $scope.loadVendorList();
                    },
                    function (err) {
                        $scope.loadVendorList();
                    });
            });
        };

        $scope.showEditBankAccount = function () {
            $scope.editBankAccount = true;
        }

        $scope.hideEditBankAccount = function () {
            $scope.editBankAccount = false;
            $scope.loadVendorList();
        }

        $scope.saveEditBankAccount = function (model) {
            confirmModalFactory('md', function () {
                settingsService.put(model,
                    function (data) {
                        $scope.loadVendorList();
                    },
                    function (err) {
                        $scope.loadVendorList();
                    });
            });
        };

        $scope.showEditVendor = function (vendor) {
            vendor.editVendor = true;
        };

        $scope.hideEditVendor = function () {
            $scope.loadVendorList();
        };

        $scope.saveEditVendor = function (model) {
            confirmModalFactory('md', function () {
                vendorService.put(model,
                    function (data) {
                        $scope.loadVendorList();
                    },
                    function (err) {
                        $scope.loadVendorList();
                    });
            });
        };

        $scope.showInvoiceDetail = function (invoice) {
            console.log(invoice);
            
            $scope.currentInvoice = invoice;
            $scope.currentInvoice.invoiceTypeName = "Nezařazeno";
            if (invoice.invoiceType) {
                $scope.currentInvoice.invoiceTypeName = "Výdej materiálu";
            } else {
                $scope.currentInvoice.invoiceTypeName = "Příjem materiálu";
            };
            $scope.currentInvoice.payTypeName = "Nezařazeno";
            if (invoice.payType) {
                $scope.currentInvoice.payTypeName = "Převodem";
            } else {
                $scope.currentInvoice.payTypeName = "Hotově";
            };

                $scope.invoiceDetail = true;
            
        };

        $scope.changeInvoicePayState = function (invoiceId, type) {
            invoiceService.changePayState(invoiceId, type,
            function (data) {
                console.log("change state ok");
                $scope.loadInvoices();
            },
            function (err) {
                console.log("change state fail", err);
                $scope.loadInvoices();
            });
        };

    });