﻿'use strict'

//token factory
dpApp.factory('authInterceptor', ['$q', '$injector', '$location', 'localStorageService',
    function ($q, $injector, $location, localStorageService) {

        var factory = {};

        factory.request = function (config) {

            config.headers = config.headers || {};

            var authData = localStorageService.get('authData');
            if (authData) {
                config.headers.Authorization = 'Bearer ' + authData.token;
            }

            return config;
        }

        factory.responseError = function (rejection) {

            if (rejection.status === 401) {
                var authService = $injector.get('authService');

                // [TODO] refresh tokens
                authService.logout();
                $location.path('/')
            }

            return $q.reject(rejection);
        }

        return factory;
    }]);

//login service
dpApp.service('authService', ['$http', '$q', 'localStorageService',
    function ($http, $q, localStorageService) {

    var _authData = {
        isAuth: false,
        userName: '',
        userId: '',
        useRefreshTokens: false // [TODO]
    }


    this.infoUser = function (id, callback) {
        $http.get('/Login/UserInfo/' + id)
            .success(function (data, status, hrds) {
                //vse ok
                console.log("info user service");
                callback(data);
            })
            .error(function () {
                //chyba
                console.log("selhalo nacteni informaci o uzivateli");
                //callback();
        });
    }

    this.currentUser = function (callback) {
        var data = localStorageService.get('authData');
        $http.get('/Login/UserInfo/' + data.userId)
            .success(function (data, status, hrds) {
                //vse ok
                callback(data);
            })
            .error(function () {
                //chyba
                console.log("selhalo nacteni informaci o prihlasenem uzivateli");
                //callback();
            });
    };

    this.signIn = function (loginData, callback) {

        var self = this;
        var data = 'username=' + loginData.Nick + '&password=' + loginData.Password + '&grant_type=password';

        var deferred = $q.defer();


        $http.post('token', data,
        {
            headers: { 'Content-Type': 'application/x-www-form-urlencoded' }
        })
            .success(function (data, status, hrds) {

                var headers = hrds();
                var id = headers['user_id'];
                localStorageService.set('authData', { token: data.access_token, username: loginData.username, userId: id})
                _authData.isAuth = true;
                _authData.userName = loginData.username;
                _authData.userId = id;
                _authData.useRefreshTokens = false;

                deferred.resolve(data);



                callback(status, id);

            })
            .error(function (data, status) {
                //self.logout();
                console.log("error login " + status);
                deferred.reject(data);
                callback(status);
            });

        return deferred.promise;
    }

    this.testAdd = function (callback) {
        $http.post('/Login/TestRoleAdd')
        .success(function (data) {
            console.log("Test add ok " + data);
            callback();
        }).error(function (data) {
            console.log("Test add fail ");
        });
    }


    this.logout = function () {
        localStorageService.remove('authData');
        _authData.isAuth = false;
        _authData.userName = '';
        _authData.userId = '';
        _authData.useRefreshTokens = false;
    }


    this.fillAuthData = function () {
        console.log("call set data auth services");
        var data = localStorageService.get('authData');
        console.log(data);
        if (data) {
            _authData.isAuth = true;
            _authData.userName = data.username;
            _authData.useRefreshTokens = false;
            _authData.userId = data.userId;
        }
        console.log(this.authentication);
    }

    this.authentication = _authData;

}]);