﻿/// <vs BeforeBuild='dev' />
function getFileNames(pathArray) {
    return pathArray.map(function (path) {
        var fileName = path.substring(path.lastIndexOf('/') + 1);
        console.log(path, fileName);
    });
}

module.exports = function (grunt) {

    var globalLessPaths = [
        "bower_components/bootstrap/less",
        "bower_components/font-awesome/less",
        "bower_components/angular-awesome-slider/src/css/less",
    ];

    var appScripts = [
        "bower_components/angular/angular.js",
        "bower_components/angular-cookies/angular-cookies.js",
        "bower_components/angular-animate/angular-animate.js",
        "bower_components/angular-bootstrap/ui-bootstrap.js",
        "bower_components/angular-bootstrap/ui-bootstrap-tpls.js",
        "bower_components/angular-route/angular-route.js",
        "bower_components/angular-sanitize/angular-sanitize.js",
        "bower_components/angular-local-storage/dist/angular-local-storage.js",
        "bower_components/angular-ui/build/angular-ui.js",
        "bower_components/restangular/dist/restangular.js",
        "bower_components/underscore/underscore.js",
        "bower_components/ng-slider/dist/ng-slider.min.js",
        "bower_components/angular-smart-table/dist/smart-table.js",
        "bower_components/ng-sortable/dist/ng-sortable.js",
        "bower_components/angular-svg-round-progressbar/build/roundProgress.js",
        "bower_components/angular-awesome-slider/dist/angular-awesome-slider.js",
        

        'Scripts/_app/*.js',
        'Scripts/_app/controllers/*.js',
        'Scripts/_app/controllers/**/*.js',
        'Scripts/_app/services/*.js',
        'Scripts/_app/factories/*.js'
    ];

    //var adminScripts = mainScriptsLegacy.concat([
    //     'Areas/Administration/js/*.js'
    //]);

    var frontendStyles = [
        'Content/*.less',
    ];

    //var adminStyles = [
    //    'Content/symfonie.admin.less',
    //    'Areas/Administration/css/*.css'
    //];

    // Project configuration.
    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        clean: {
            dist: { src: ["dist"], force: true }
        },

        less: {
            dist: {
                options: {
                    cleancss: true,
                    minify: true,
                    paths: globalLessPaths
                },
                files: {
                    "dist/styles/dp.min.css": frontendStyles,
                    //"dist/styles/symfonie.admin.min.css": adminStyles
                }
            },
            dev: {
                options: {
                    cleancss: false,
                    paths: globalLessPaths
                },
                files: {
                    "dist/styles/dp.css": frontendStyles,
                    //"dist/styles/symfonie.admin.css": adminStyles
                }
            }
        },
        uglify: {
            dist: {
                options: {
                    banner: '/*! Zemedelci (build on: <%= grunt.template.today("yyyy-mm-dd hh:MM") %>) */\n',
                    mangle: false,
                    compress: false,
                    beautify: false,
                    //paths: appScripts
                },
                files: {
                    'dist/scripts/dp.min.js': appScripts,
                    //'dist/scripts/wt.admin.min.js': adminScripts
                }
            }
        },

        copy: {
            dev: {
                files: [
                    {
                        expand: true,
                        flatten: true,
                        src: appScripts,
                        dest: 'dist/scripts/frontend',
                        filter: 'isFile'
                    }/*,
                    {
                        expand: true,
                        flatten: true,
                        src: adminScripts,
                        dest: 'dist/scripts/admin',
                        filter: 'isFile'
                    }*/
                ]
            },
            all: {
                files: [
                    {
                        expand: true,
                        flatten: true,
                        src: [
                            'bower_components/bootstrap/fonts/**',
                            'bower_components/font-awesome/fonts/**'
                        ],
                        dest: 'dist/fonts/',
                        filter: 'isFile'
                    },
                ]
            }
        },

        injector: {
            options: {
                lineEnding: '\r\n',
                addRootSlash: true
            },
            dev: {
                files: {
                    'Views/Shared/_Layout.cshtml': appScripts.concat(['dist/styles/dp.css', 'dist/scripts/dp.js']),
                    //'Areas/Administration/Views/Shared/_Layout.cshtml': adminScripts.concat(["dist/styles/working-timer.admin.css"])
                }
            },
            dist: {
                files: {
                    'Views/Shared/_Layout.cshtml': ['dist/styles/dp.min.css', 'dist/scripts/dp.min.js'],
                    //'Areas/Administration/Views/Shared/_Layout.cshtml': ['dist/styles/working-timer.admin.min.css', 'dist/scripts/working-timer.admin.min.js']
                }
            }
        },

        watch: {
            less: {
                files: ['Content/*.less'],
                tasks: ['less'],
                options: {
                    spawn: false,
                },
            },
            scripts: {
                files: ['Scripts/**/*.js'],
                tasks: ['uglify'],
                options: {
                    spawn: false,
                },
            }
        }

    });

    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-injector');

    // tasks    
    grunt.registerTask('dist', ['clean:dist', 'copy:all', 'less:dist', 'uglify:dist', 'injector:dist']);
    grunt.registerTask('dev', ['clean:dist', 'copy', 'less:dev', 'uglify:dist', 'injector:dev']);
    grunt.registerTask('default', ['dev']);

    //tasks for MsBuild Node Tools: https://github.com/kmees/MSBuild.NodeTools   
    //for each configuration one grunt task
    grunt.registerTask('build-Debug', ['dev']);
    grunt.registerTask('build-Release', ['dist']);

    //fallback for MsBuild if target configuration is not explicitly set
    grunt.registerTask('build', ['dist']);
    grunt.registerTask('build-', ['dist']);

};