﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Web.Http;
using Microsoft.Owin.Security.OAuth;
using Newtonsoft.Json.Serialization;
using Owin;
using System.Web.Http.Routing;
using System.Web.Http.Controllers;
using business;
using LightInject;
using Data.Models;
using bussines.Services;

namespace WebAppDp
{
    public partial class Startup
    {
        public void ConfigureWebApi(IAppBuilder app)
        {

            var config = new HttpConfiguration();
            var container = new ServiceContainer();

            // IoC container
            container.RegisterApiControllers();
            container.EnableWebApi(config);


            // services
            container.Register<CompanyService>();
            container.Register<SoilBlockService>();
            container.Register<StockItemService>();
            container.Register<VendorService>();
            container.Register<PartSoilBlockService>();
            container.Register<WcatToMachineService>();
            container.Register<HarvestedCropService>();
            container.Register<CostPlanService>();
            container.Register<BusinessRecordService>();
            container.Register<BankAccountService>();
            container.Register<CropRotationPlanService>();
            container.Register<InvoiceService>();

            container.Register<IService<Address>, AddressService>();
            container.Register<IService<Contact>, ContactService>();
            container.Register<IService<Company>, CompanyService>();
            container.Register<IService<User>, UserService>();
            container.Register<IService<Vendor>, VendorService>();


            // Use camel case for JSON data.
            config.Formatters.JsonFormatter.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();

            // Web API routes
            config.MapHttpAttributeRoutes(new InheritedRouteProvider());

            config.Routes.MapHttpRoute(
                name: "DefaultApi",
                routeTemplate: "api/{controller}/{id}",
                defaults: new { id = RouteParameter.Optional }
            );

            app.UseWebApi(config);
        }

        public class InheritedRouteProvider : DefaultDirectRouteProvider
        {
            protected override IReadOnlyList<IDirectRouteFactory> GetActionRouteFactories(HttpActionDescriptor actionDescriptor)
            {
                return actionDescriptor.GetCustomAttributes<IDirectRouteFactory>(true);
            }
        }
    }
}
