﻿using business;
using bussines.Services;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.Interfaces;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    public abstract class BaseController<TModel, TViewModel> : ApiController
        where TModel : BaseModel
        where TViewModel : IViewModel
    {
        private IService<TModel> _service { get; set; }

        public BaseController()
        {
            _service = new BaseService<TModel>();
        }

        public BaseController(IService<TModel> service)
        {
            this._service = service;
        }

        [Route, HttpGet]
        public virtual async Task<IEnumerable<TViewModel>> GetAll()
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;
            var models = DataToModels(_service.GetAll(companyId).Result);
            return await Task.FromResult<IEnumerable<TViewModel>>(models);
        }

        [Route("{id:long}"), HttpGet]
        public virtual async Task<TViewModel> Get(long id)
        {
            var model = DataToModel(_service.GetById(id).Result);
            return await Task.FromResult<TViewModel>(model);
        }

        [Route, HttpPost]
        public virtual async Task<long> Post(TViewModel model)
        {
            var data = ModelToData(model);
            return await _service.Put(data);
        }

        [Route("{id:long}"), HttpPost]
        public virtual async Task<long> Put(long id, TViewModel model)
        {
            var data = ModelToData(model);
            return await _service.Put(data);
        }

        [Route("{id:long}"), HttpDelete]
        public virtual async Task Delete(long id)
        {
            await _service.Delete(id);
        }

        protected virtual IEnumerable<TViewModel> DataToModels(IEnumerable<TModel> models)
        {
            return models.Select(m => DataToModel(m)).ToList();
        }

        protected abstract TModel ModelToData(TViewModel model);

        protected abstract TViewModel DataToModel(TModel data);
    }
}
