﻿using bussines.Services;
using Data;
using Data.Models;
using Data.Repositories;
using WebAppDp.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Web.Helpers;
using Data.Interfaces;
using System.Threading;
using System.Threading.Tasks;
using WebAppDp.Helpers;
using business;
using System.Web.Http;

namespace WebAppDp.Controllers
{
    [Authorize]
    [RoutePrefix("SoilBlock")]
    public class SoilBlockController : BaseController<SoilBlock, SoilBlockViewModel>
    {
        public SoilBlockService Service { get; set; }

        public SoilBlockController(SoilBlockService service)
            : base(service)
        {
            Service = service;
        }

        [Route("get/{id}/year/{year}"), HttpGet]
        public virtual async Task<SoilBlockViewModel> Get(long id, long year)
        {
            var model = DataToModel(Service.GetById(id).Result, year);
            return await Task.FromResult<SoilBlockViewModel>(model);
        }

        [Route("create"), HttpPost]
        public async Task<IEnumerable<PartSoilBlockViewModel>> PostAndGetParts(SoilBlockViewModel model)
        {
            var data = Service.PutAndGetParts(this.ModelToData(model), model.CountPartsSoilBlocks).Result;
            return await Task.Run(() => PartSoilBlockHelper.DataToModels(data));
        }

        [Route("all/{year}"), HttpGet]
        public async Task<IEnumerable<SoilBlockViewModel>> GetAll(long year)
        {
            long companyId = IdentityHelper.CurrentUser.CompanyId;

            return await Task.Run(() =>
            {
                var data = new List<SoilBlockViewModel>();
                var soilBlocks = Service.GetAll(companyId).Result;
                var results = Service.GetBlocksPartsCount(soilBlocks.Select(b => b.Id), year).Result;

                foreach (var block in soilBlocks)
                {

                    var model = DataToModel(block);
                    if (results.ContainsKey(block.Id))
                    {
                        model.CountPartsSoilBlocks = (int)results[block.Id];
                    }
                    else
                    {
                        model.CountPartsSoilBlocks = 0;
                    }

                    data.Add(model);
                }

                return data;
            });
        }

        protected override SoilBlock ModelToData(SoilBlockViewModel model)
        {
            return new SoilBlock
            {
                CompanyId = IdentityHelper.CurrentUser.CompanyId,
                Area = model.Area,
                CreatedAt = model.CreatedAt,
                IdentificationNumber = model.IdentificationNumber,
                Name = model.Name,
                N = model.N,
                K = model.K,
                S = model.S,
                P = model.P,
                Id = model.Id
            };
        }

        protected SoilBlockViewModel DataToModel(SoilBlock data, long year = 0)
        {
            PartSoilBlockService partSoilBlockService = new PartSoilBlockService();
            return new SoilBlockViewModel
            {
                CompanyId = data.CompanyId,
                Id = data.Id,
                Area = data.Area,
                CreatedAt = data.CreatedAt,
                IdentificationNumber = data.IdentificationNumber,
                Name = data.Name,
                N = data.N,
                K = data.K,
                S = data.S,
                P = data.P,
                PartSoilBlocks = new PartSoilBlockController(partSoilBlockService).GetPartBySoilBlockId(data.Id, year).Result
            };
        }

        protected override SoilBlockViewModel DataToModel(SoilBlock data)
        {
            return new SoilBlockViewModel
            {
                CompanyId = data.CompanyId,
                Id = data.Id,
                Area = data.Area,
                CreatedAt = data.CreatedAt,
                IdentificationNumber = data.IdentificationNumber,
                Name = data.Name,
                N = data.N,
                K = data.K,
                S = data.S,
                P = data.P,
                PartSoilBlocks = null
            };
        }
    }
}