﻿using bussines.Services;
using Data.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using WebAppDp.Helpers;
using WebAppDp.ViewModels;

namespace WebAppDp.Controllers
{
    [RoutePrefix("Vendor")]
    public class VendorController : BaseController<Vendor,VendorViewModel>
    {
        VendorService Service { get; set; }

        public VendorController(VendorService service)
            : base(service)
        {
            Service = service;
        }

        [Route("getProduction"), HttpGet]
        public async Task<Vendor> GetProductionVendor()
        {
            return await Service.GetProductionVendor(IdentityHelper.CurrentUser.CompanyId);
        }
        protected override Vendor ModelToData(VendorViewModel model)
        {
            long compId = 0;
            if (model.Production)
            {
                compId = model.CompanyId;
            }
            else 
            {
                compId = IdentityHelper.CurrentUser.CompanyId;
            }

            return new Vendor {
                AddressId = model.AddressId,
                CompanyId = compId,
                ContactId = model.ContactId,
                CreatedAt = model.CreatedAt,
                Dic = model.Dic,
                Ico = model.Ico,
                Id = model.Id,
                Name = model.Name,
                Production = model.Production,
                BankAccountId = model.BankAccountId
            };
        }

        protected override VendorViewModel DataToModel(Vendor data)
        {
            bool editableStatus = true;
            Address _address = new Address();
            Contact _contact = new Contact();
            BankAccount _bankAccount = new BankAccount();

            AddressService AddressService = new AddressService();
            ContactService ContactService = new ContactService();
            BankAccountService BankAccountService = new BankAccountService();

            if (data.AddressId > 0) {
                _address = AddressService.GetById(data.AddressId).Result;
            };

            if (data.ContactId > 0) {
                _contact = ContactService.GetById(data.ContactId).Result;
            };

            if (data.BankAccountId > 0) {
                _bankAccount = BankAccountService.GetById(data.BankAccountId).Result;
            }

            if (data.Name == "Vlastní výroba")
            {
                editableStatus = false;
            };
            return new VendorViewModel
            {
                AddressId = data.AddressId,
                CompanyId = data.CompanyId,
                ContactId = data.ContactId,
                CreatedAt = data.CreatedAt,
                Dic = data.Dic,
                Ico = data.Ico,
                Id = data.Id,
                Name = data.Name,
                Production = data.Production,
                editable = editableStatus,
                BankAccountId = data.BankAccountId,
                AddresModel = _address,
                ContactModel = _contact,
                BankAccountModel = _bankAccount
            };
        }
    }
}