﻿using business;
//using LightInject;
using Data.Interfaces;
using Data.Models;
using Data.Repositories;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;
using bussines.Exceptions;

namespace bussines.Services
{
    public class BaseService<T> : IService<T> where T : BaseModel
    {
        public IValidator<T> Validator
        {
            get;
            set;
        }

        public BaseService()
        {
            Repository = new BaseRepository<T>();
        }

        public IRepository<T> Repository { get; set; }

        public async virtual Task<IEnumerable<T>> GetAll(long companyId)
        {
            return await Repository.GetAll(companyId);
        }

        public async virtual Task<T> GetById(long id)
        {
            return await Repository.GetById(id);
        }

        public async virtual Task Delete(long id)
        {
            await Repository.Delete(id);
        }

        public async virtual Task<long> Put(T model)
        {
            if (Validator != null)
            {
                var errors = Validator.Validate(model);
                if (errors.Count() > 0)
                {
                    throw new ValidationException(string.Join("\n", errors));
                }
            }

            if (model.Id != 0)
            {
                return await Repository.Update(model);
            }
            else
            {
                return await Repository.Create(model);
            }

        }
    }
}